/*
 * Copyright (c) 2010 Ryan Vogt <vogt@cs.ualberta.ca>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/**
 * The <code>APPlanner</code> class is responsible for comparing all the battles
 * you plan to fight against the GF abilities that you wish to learn.
 */
public class APPlanner
{
    /**
     * Runs the <code>APPlanner</code> application using the first argument
     * as the GF file and the second argument as the battle file.
     *
     * @param args the arguments passed from the Java VM.
     */
    public static void main(String[] args)
    {
	/* Verify the command line arguments */
	if (args.length != 2) {
	    System.err.println("Usage: APPlanner GF_FILE BATTLE_FILE");
	    System.exit(-1);
	}

	/* Execute the user-specified simulation */
	Simulation sim = APPlanner.parseInputFiles(args[0], args[1]);
	if (sim == null)
	    System.exit(-1);
	sim.execute();
    }

    /**
     * Parses the two user input files and returns the resulting simulation.
     *
     * @param gfFile the name of the GF file.
     * @param battleFile the name of the battle file.
     * @return the resulting simulation from the two input files, or
     * <code>null</code> if there is an error in the input files.
     */
    private static Simulation parseInputFiles(String gfFile, String battleFile)
    {
	Simulation simulation = new Simulation();

	/* Parse the GF file */
	Parser gfParser = new Parser();
	gfParser.add(new AbilityCommand());
	gfParser.add(new GFCommand());
	try {
	    gfParser.parse(gfFile, simulation);
	}
	catch (ParserException pe) {
	    System.err.println("Error in GF file: " + pe.getMessage());
	    return null;
	}

	/* Parse the battle file */
	Parser battleParser = new Parser();
	battleParser.add(new EarnCommand());
	battleParser.add(new ExcludedCommand());
	battleParser.add(new GetCommand());
	battleParser.add(new NoMenuCommand());
	try {
	    battleParser.parse(battleFile, simulation);
	}
	catch (ParserException pe) {
	    System.err.println("Error in battle file: " + pe.getMessage());
	    return null;
	}

	return simulation;
    }
}