/*
 * Copyright (c) 2010 Ryan Vogt <vogt@cs.ualberta.ca>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/**
 * The <code>Ability</code> class represents an ability that a Guardian Force
 * can learn.
 */
public class Ability
{
    /**
     * The name of the ability.
     */
    private String name;

    /**
     * The current amount of AP earned towards the ability.
     */
    private int earned;

    /**
     * The amount of AP required to learn the ability.
     */
    private int required;

    /**
     * Creates a new ability with the given name and required AP to learn, with
     * no AP yet earned toward it.
     *
     * @param name the name of the ability.
     * @param ap the amount of AP required.
     * @throws IllegalArgumentException if <code>ap</code> is less than or
     * equal to zero.
     */
    public Ability(String name, int ap)
    {
	if (ap <= 0)
	    throw new IllegalArgumentException("Invalid ability AP");
	this.name = name;
	this.required = ap;
	this.earned = 0;
    }

    /**
     * Returns the name of the ability.
     *
     * @return the name of the ability.
     */
    public String getName()
    {
	return this.name;
    }

    /**
     * Returns the amount of AP required to learn the ability.
     *
     * @return the amount of AP requird to learn the ability.
     */
    public int getRequiredAP()
    {
	return this.required;
    }

    /**
     * Returns the amount of AP earned so far towards the ability.
     *
     * @return the amount of AP earned so far.
     */
    public int getEarnedAP()
    {
	return this.earned;
    }

    /**
     * Adds the given amount of AP to the amount earned toward this ability.
     *
     * @param ap the amount of AP to add.
     * @return <code>true</code> if the earned AP became greater than or equal
     * to the required AP because of this addition (and was not previously
     * greater than or equal to the earned AP), otherwise <code>false</code>.
     * @throws IllegalArgumentException if <code>ap</code> is negative.
     */
    public boolean earn(int ap)
    {
	if (ap < 0)
	    throw new IllegalArgumentException("Invalid AP earning");
	boolean could = (this.earned < this.required);
	this.earned += ap;
	return (could && (this.earned >= this.required));
    }

    /**
     * Returns whether this ability has been learned.
     *
     * @return <code>true</code> if the amount of earned AP meets or exceeds
     * the amount of required AP, otherwise <code>false</code>.
     */
    public boolean isLearned()
    {
	return (this.earned >= this.required);
    }
}