/*
 * Copyright (c) 2010 Ryan Vogt <vogt@cs.ualberta.ca>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/**
 * The <code>Battle</code> class represents a single battle in Final Fantasy
 * VIII that awards AP to Guardian Forces.
 */
public class Battle
{
    /**
     * The name of the battle.
     */
    private String name;

    /**
     * The amount of AP awarded by the battle.
     */
    private int ap;

    /**
     * Whether you can access the menu before this battle.
     */
    private boolean menuBefore;

    /**
     * Creates a new battle with the given name that awards the given amount of
     * AP, where you have the ability to access the menu before this battle.
     *
     * @param name the name of the battle.
     * @param ap the amount of AP awarded.
     * @throws IllegalArgumentException if <code>ap</code> is negative.
     */
    public Battle(String name, int ap)
    {
        if (ap < 0)
            throw new IllegalArgumentException("Invalid battle AP reward");
        this.name = name;
        this.ap = ap;
	this.menuBefore = true;
    }

    /**
     * Returns the name of the battle.
     *
     * @return the name of the battle.
     */
    public String getName()
    {
        return this.name;
    }

    /**
     * Returns the amount of AP awarded by the battle.
     *
     * @return the amount of AP from the battle.
     */
    public int getAP()
    {
        return this.ap;
    }

    /**
     * Returns whether or not you can access the menu before this battle
     * begins.
     *
     * @return <code>true</code> if you can access the menu before this battle,
     * otherwise <code>false</code>.
     */
    public boolean canMenuBefore()
    {
	return this.menuBefore;
    }

    /**
     * Sets whether you can access the menu before this battle.
     *
     * @param menu the new value for whether you can access the menu before
     * this battle.
     */
    public void setMenuBefore(boolean menu)
    {
	this.menuBefore = menu;
    }
}