/*
 * Copyright (c) 2010 Ryan Vogt <vogt@cs.ualberta.ca>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

import java.util.*;

/**
 * The <code>Command</code> class represents a type of command that can appear
 * in one of the input files to the AP Planning Program.
 */
public abstract class Command
{
    /**
     * The single word that identifies this type of command.
     */
    protected String word;

    /**
     * Creates a new <code>Command</code> for commands that begin with the
     * given single word.
     *
     * @param word the first word of this type of command.
     */
    public Command(String word)
    {
	this.word = word;
    }

    /**
     * Returns the single word that starts this command.
     *
     * @return the single identifying word.
     */
    public String getWord()
    {
	return this.word;
    }

    /**
     * Attempts to match the given line from an input file against this
     * command template.  If the line matches this command, execute the command
     * as specified in the user input file.
     *
     * @param line the input file line, with any comments and leading or
     * trailing whitespace removed.
     * @param simulation the simulation on which to execute the parsed command.
     * @return <code>true</code> if the user input line resulted in a command
     * being executed on the simulation, or <code>false</code> if the input
     * line does not match this type of command.
     * @throws IllegalArgumentException if the given line is empty.
     * @throws ParserException if the given line matches the single word
     * identifier of this command, but there is an error parsing the rest
     * of the line.
     */
    public boolean parse(String line, Simulation simulation)
	throws ParserException
    {
	Scanner lineScanner = new Scanner(line);
	if (lineScanner.hasNext() == false)
	    throw new IllegalArgumentException("Empty line");
	String word = lineScanner.next();

	if (word.equals(this.word)) {
	    String rest = "";
	    lineScanner.useDelimiter("$");
	    if (lineScanner.hasNext())
		rest = lineScanner.next().trim();
	    this.parseRest(rest, simulation);
	    return true;
	}
	else
	    return false;
    }

    /**
     * Parses the remainder of the input line after the single identifying
     * word.
     *
     * @param rest the remainder of the line.
     * @param simulation the simulation on which to execute the parsed command.
     * @throws ParserException if the remainder of the line does not match the
     * expected format.
     */
    protected abstract void parseRest(String rest, Simulation simulation)
	throws ParserException;
}