/*
 * Copyright (c) 2010 Ryan Vogt <vogt@cs.ualberta.ca>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

import java.util.*;

/**
 * The <code>CommandValued</code> class represents a command in which a single
 * identifying word is followed by an integer and then a string that may
 * contain spaces.
 */
public abstract class CommandValued extends Command
{
    /**
     * The minimum legal value for the integer that follows the identifying
     * word.
     */
    private int minValue;

    /**
     * Creates a new <code>CommandValued</code> with the given identifying word
     * and minimum legal value for the integer component.
     *
     * @param word the identifying word.
     * @param minValue the minimum legal value.
     */
    public CommandValued(String word, int minValue)
    {
	super(word);
	this.minValue = minValue;
    }

    /**
     * Parses the remainder of the input line after the single identifying
     * word.
     *
     * @param rest the remainder of the line.
     * @param simulation the simulation on which to execute the parsed command.
     * @throws ParserException if the remainder of the line does not match the
     * expected format.
     */
    protected void parseRest(String rest, Simulation simulation)
	throws ParserException
    {
	Scanner lineScanner = new Scanner(rest);
	if (lineScanner.hasNextInt() == false)
	    throw new ParserException("Missing integer for " + this.word);
	int value = lineScanner.nextInt();
	if (value < this.minValue)
	    throw new ParserException("Invalid value for " + this.word);
	lineScanner.useDelimiter("$");
	if (lineScanner.hasNext() == false)
	    throw new ParserException("Missing argument for " + this.word);
	String argument = lineScanner.next().trim();
	this.execute(value, argument, simulation);
    }

    /**
     * Executes the user command on the given simulation.
     *
     * @param value the integer argument to the command.
     * @param argument the string argument to the command.
     * @param simulation the simulation on which to execute the command.
     * @throws ParserException if the given arguments are illegal.
     */
    protected abstract void execute(int value, String argument,
				    Simulation simulation)
	throws ParserException;
}