/*
 * Copyright (c) 2010 Ryan Vogt <vogt@cs.ualberta.ca>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/**
 * The <code>ExcludedCommand</code> represents a command in a battle input
 * file that specifies a GF that was excluded from the previously defined
 * battle.
 */
public class ExcludedCommand extends CommandString
{
    /**
     * Creates a new <code>ExcludedCommand</code> to be added to a parser.
     */
    public ExcludedCommand()
    {
	super("excluded");
    }

    /**
     * Executes the excluded command on the given simulation.
     *
     * @param argument the name of the GF.
     * @param simulation the simulation on which to execute the command.
     * @throws ParserException if the GF does not exist in the simulation or
     * if it has not yet been received, or if there are not yet any battles in
     * the simulation, or if the exclusion command is a duplicate.
     */
    protected void execute(String argument, Simulation simulation)
	throws ParserException
    {
	GuardianForce gf = simulation.getGF(argument);
	if (gf == null)
	    throw new ParserException("Invalid GF name: " + argument);

	int fromBattle = simulation.numBattles() - 1;
	if (fromBattle < 0)
	    throw new ParserException("No battle from which to exclude " +
				      argument);

	if (gf.acquiredBy(fromBattle) == false)
	    throw new ParserException("Excluding " + argument +
				      " when not yet received");

	if (gf.inBattle(fromBattle) == false)
	    throw new ParserException("Duplicate exclusion of " + argument);

	gf.excludeFrom(fromBattle);
    }
}