/*
 * Copyright (c) 2010 Ryan Vogt <vogt@cs.ualberta.ca>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

import java.util.*;

/**
 * The <code>GuardianForce</code> class represents a GF in Final Fantasy VIII
 * that can learn abilities.
 */
public class GuardianForce
{
    /**
     * The GF's name.
     */
    private String name;

    /**
     * The current ability that the GF is learning, or <code>null</code> to
     * indicate that the GF is not learning any ability.
     */
    private Ability current;

    /**
     * The subsequent abilities that the GF wants to learn.
     */
    private LinkedList<Ability> next;

    /**
     * The number of battles after which this GF is acquired, or
     * <code>-1</code> to indicate that this GF is never acquired.
     */
    private int acquiredAfter;

    /**
     * The battles after which this GF is acquired from which this GF is
     * excluded, where battle numbers are indexed from zero.
     */
    private Set<Integer> excludedFrom;

    /**
     * The AP earned with no ability to direct it to.
     */
    private int unused;

    /**
     * The total AP used to "overlearn" an ability.
     */
    private int wasted;

    /**
     * The number of abilities learned.
     */
    private int learned;

    /**
     * Creates a new Guardian Force with the given name that has no abilities
     * to learn.
     *
     * @param name the name of the Guardian Force.
     */
    public GuardianForce(String name)
    {
        this.name = name;
	this.current = null;
        this.next = new LinkedList<Ability>();
	this.acquiredAfter = -1;
	this.excludedFrom = new HashSet<Integer>();
        this.unused = 0;
        this.wasted = 0;
        this.learned = 0;
    }

    /**
     * Returns the name of this Guardian Force.
     *
     * @return the GF's name.
     */
    public String getName()
    {
        return this.name;
    }

    /**
     * Returns the ability that is currently being learned, or
     * <code>null</code> if no ability is being learned.
     *
     * @return the ability currently being learned.
     */
    public Ability getCurrent()
    {
	return this.current;
    }

    /**
     * Appends the given ability to the end of the queue of abilities to learn.
     * Note that under no circumstances does this function affect the ability
     * currently being learned.
     *
     * @param ability the ability to append.
     * @throws IllegalArgumentException if the given ability is null, or if
     * there is an ability of the same name currently being learned or in the
     * queue.
     */
    public void append(Ability ability)
    {
	if (ability == null)
	    throw new IllegalArgumentException("Null ability");
	String name = ability.getName();
	if (this.current != null && this.current.getName().equals(name))
	    throw new IllegalArgumentException("Duplicate ability name");
	for (Ability ab : this.next) {
	    if (ab.getName().equals(name))
		throw new IllegalArgumentException("Duplicate ability name");
	}
	this.next.add(ability);
    }

    /**
     * Returns whether there are abilities on the queue of future abilities to
     * learn.
     *
     * @return <code>true</code> if there are additional abilities to learn,
     * otherwise <code>false</code>.
     */
    public boolean isNextAbility()
    {
	return !(this.next.isEmpty());
    }

    /**
     * Replaces the ability currently being learned with the next ability on
     * the queue, or <code>null</code> if there are no abilities on the queue.
     */
    public void nextAbility()
    {
	this.current = this.next.poll();
    }

    /**
     * Sets the number of battles that must be fought before this GF is
     * acquired.  A negative value indicates that this GF is never acquired.  A
     * value of zero indicates that this GF is acquired before the first battle
     * of the game.
     *
     * @param battles the number of battles that must be fought before this
     * GF is acquired, or a negative value to indicate never.
     * @throws IllegalArgumentException if there are any battles that this GF
     * is explicitly excluded from that are before the new acquisition point,
     * or if there are any battles that this GF is explicitly excluded from and
     * this GF is now not acquired (see {@link #excludeFrom}).
     */
    public void setAcquiredAfter(int battles)
    {
	if (battles < 0) {
	    battles = -1;
	    if (this.excludedFrom.isEmpty() == false)
		throw new IllegalArgumentException("Exclusion mismatch");
	}
	else {
	    for (Integer excl : this.excludedFrom) {
		if (excl < battles)
		    throw new IllegalArgumentException("Exclusion mismatch");
	    }
	}
	this.acquiredAfter = battles;
    }

    /**
     * Explicitly excludes this GF from the given battle, where battles are
     * indexed from zero.
     *
     * @param battle the battle from which to explicitly exclude this GF.
     * @throws IllegalArgumentException if the given battle occurs before the
     * point this GF is acquired, or if this GF is never acquired (see
     * {@link #setAcquiredAfter}), or if the given battle index is negative.
     */
    public void excludeFrom(int battle)
    {
	if (battle < 0)
	    throw new IllegalArgumentException("Invalid battle index");
	if (this.acquiredAfter == -1 || battle < this.acquiredAfter)
	    throw new IllegalArgumentException("Exclusion mismatch");
	this.excludedFrom.add(battle);
    }

    /**
     * Returns whether this GF is acquired before any battles occur.
     *
     * @return <code>true</code> if this GF is acquried in the opening,
     * otherwise <code>false</code>.
     */
    public boolean acquiredInOpening()
    {
	return (this.acquiredAfter == 0);
    }

    /**
     * Returns whether this GF is acquired at all.
     *
     * @return <code>true</code> if this GF is acquired, otherwise
     * <code>false</code>.
     */
    public boolean isAcquired()
    {
	return (this.acquiredAfter != -1);
    }

    /**
     * Tests whether this GF has been acquired by the start of the given
     * battle, where battles are indexed from zero.
     *
     * @param battle the battle index.
     * @return <code>true</code> if this GF has been acquired by the start of
     * the given battle, otherwise <code>false</code>.
     * @throws IllegalArgumentException if the given battle index is negative.
     */
    public boolean acquiredBy(int battle)
    {
	if (battle < 0)
	    throw new IllegalArgumentException("Invalid battle index");
	if (this.acquiredAfter == -1 || battle < this.acquiredAfter)
	    return false;
	return true;
    }

    /**
     * Tests whether this GF is acquired during the given battle, where battles
     * are indexed from zero.
     *
     * @param battle the battle index.
     * @return <code>true</code> if this GF is acquired during the given
     * battle.
     * @throws IllegalArgumentException if the given battle index is negative.
     */
    public boolean acquiredIn(int battle)
    {
	if (battle < 0)
	    throw new IllegalArgumentException("Invalid battle index");
	return (battle + 1 == this.acquiredAfter);
    }

    /**
     * Tests whether this GF participates in the given battle, where battles
     * are indexed from zero.
     *
     * @param battle the battle index.
     * @return <code>true</code> if this GF is in the given battle, otherwise
     * <code>false</code>.
     * @throws IllegalArgumentException if the given battle index is negative.
     */
    public boolean inBattle(int battle)
    {
	if (battle < 0)
	    throw new IllegalArgumentException("Invalid battle index");
	if (this.acquiredAfter == -1 || battle < this.acquiredAfter)
	    return false;
	Integer lookup = Integer.valueOf(battle);
	return !(this.excludedFrom.contains(lookup));
    }

    /**
     * Adds the given amount of AP to the tally of unused AP.
     *
     * @param ap the amount of AP to add.
     * @throws IllegalArgumentException if the amount of AP is negative.
     */
    public void addUnused(int ap)
    {
	if (ap < 0)
	    throw new IllegalArgumentException("Invalid AP amount");
	this.unused += ap;
    }

    /**
     * Returns the current tally of unused AP by this GF, meaning AP earned
     * when the GF was not aspiring to learn anything.
     *
     * @return the amount of unused AP.
     */
    public int getUnused()
    {
	return this.unused;
    }

    /**
     * Adds the given amount of AP to the tally of wasted AP.
     *
     * @param ap the amount of AP to add.
     * @throws IllegalArgumentException if the amount of AP is negative.
     */
    public void addWasted(int ap)
    {
	if (ap < 0)
	    throw new IllegalArgumentException("Invalid AP amount");
	this.wasted += ap;
    }

    /**
     * Returns the current tally of wasted AP by this GF, meaning AP that goes
     * to waste due to overlearning or having not yet dequeued an ability to
     * learn.
     *
     * @return the amount of wasted AP.
     */
    public int getWasted()
    {
	return this.wasted;
    }

    /**
     * Increments the number of abilities learned by this GF by one.
     */
    public void incrementLearned()
    {
	this.learned++;
    }

    /**
     * Returns the number of abilities learned by this GF.
     *
     * @return the number of abilities learned.
     */
    public int getLearned()
    {
	return this.learned;
    }

    /**
     * Returns the amount of AP required to learn the current ability and all
     * the remaining abilities in the queue.
     *
     * @return the total amount of AP still required.
     */
    public int getTotalRequired()
    {
	int ret = 0;
	if (this.current != null && this.current.isLearned() == false)
	    ret += this.current.getRequiredAP() - this.current.getEarnedAP();
	for (Ability ab : this.next) {
	    if (ab.isLearned())
		continue;
	    ret += ab.getRequiredAP() - ab.getEarnedAP();
	}
	return ret;
    }
}