/*
 * Copyright (c) 2010 Ryan Vogt <vogt@cs.ualberta.ca>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

import java.io.*;
import java.util.*;

/**
 * The <code>Parser</code> class is used to parse an input file to the AP
 * Planning program.
 */
public class Parser
{
    /**
     * The character that indicates the rest of the line is a comment.
     */
    private static final char COMMENT_CHAR = '#';

    /**
     * All of the valid command types that can appear in the file being parsed.
     */
    private List<Command> valid;

    /**
     * Creates a new <code>Parser</code> with no valid commands.
     */
    public Parser()
    {
	this.valid = new ArrayList<Command>();
    }

    /**
     * Adds the given command to the list of valid commands that can appear in
     * the file being parsed.
     *
     * @param command the command to add.
     * @throws IllegalArgumentException if the given command is
     * <code>null</code>, or if the single identifying word of the given
     * command matches a command already added to this parser.
     */
    public void add(Command command)
    {
	if (command == null)
	    throw new IllegalArgumentException("Null command");
	String newWord = command.getWord();
	for (Command c : this.valid) {
	    if (c.getWord().equals(newWord))
		throw new IllegalArgumentException("Duplicate command word");
	}
	this.valid.add(command);
    }

    /**
     * Parses the given input file, and executes all of the commands generated
     * on the given simulation.
     *
     * @param filename the file to parse.
     * @param simulation the target of the commands in the given file.
     * @throws ParserException if there is an error with the input file format.
     */
    public void parse(String filename, Simulation simulation)
	throws ParserException
    {
	/* Open the input file */
	Scanner scanner = null;
	try {
	    scanner = new Scanner(new File(filename));
	}
	catch (IOException ioe) {
	    throw new ParserException("Error opening file: " + filename);
	}

	while (scanner.hasNextLine()) {
	    String line = scanner.nextLine();

	    /* Trim off comments */
	    int comment = line.indexOf(Parser.COMMENT_CHAR);
	    if (comment != -1)
		line = line.substring(0, comment);
	    line = line.trim();
	    if (line.isEmpty())
		continue;

	    /* Test if it matches any command */
	    boolean match = false;
	    for (Command command : this.valid) {
		match = command.parse(line, simulation);
		if (match == true)
		    break;
	    }
	    if (match == false)
		throw new ParserException("Unknown line: " + line);
	}
    }
}