/*
 * Copyright (c) 2010 Ryan Vogt <vogt@cs.ualberta.ca>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

import java.util.*;

/**
 * The <code>Simulation</code> class represents a simulated run through all of
 * Final Fantasy VIII, keeping track of when abilities are learned.
 */
public class Simulation
{
    /**
     * All of the GFs involved in the simulation.
     */
    private List<GuardianForce> gfs;

    /**
     * All of the battles in the simulation.
     */
    private List<Battle> battles;

    /**
     * Creates a new <code>Simulation</code> with no Guardian Forces and no
     * battles.
     */
    public Simulation()
    {
	this.gfs = new ArrayList<GuardianForce>();
	this.battles = new ArrayList<Battle>();
    }

    /**
     * Returns the number of Guardian Forces in the simulation.
     *
     * @return the numbers of GFs.
     */
    public int numGFs()
    {
	return this.gfs.size();
    }

    /**
     * Returns the Guardian Force at the given index.
     *
     * @param index the index of the GF to return.
     * @return the GF at the given index.
     * @throws ArrayIndexOutOfBoundsException if the given index is negative or
     * greater than or equal to the number of GFs.
     */
    public GuardianForce getGF(int index)
    {
	return this.gfs.get(index);
    }

    /**
     * Returns the Guardian Force with the given name, or <code>null</code> if
     * it does not exist.
     *
     * @param name the name of the GF to return.
     * @return the GF with that name or <code>null</code>.
     */
    public GuardianForce getGF(String name)
    {
	for (GuardianForce gf : this.gfs) {
	    if (gf.getName().equals(name))
		return gf;
	}
	return null;
    }

    /**
     * Adds the given Guardian Force to the end of the list of Guardian Forces
     * in this simulation.
     *
     * @param gf the GF to add.
     * @throws IllegalArgumentException if a GF with the same name already
     * exists.
     */
    public void add(GuardianForce gf)
    {
	if (this.getGF(gf.getName()) != null)
	    throw new IllegalArgumentException("Duplicate GF name");
	this.gfs.add(gf);
    }

    /**
     * Returns the number of battles in the simulation.
     *
     * @return the numbers of battles.
     */
    public int numBattles()
    {
	return this.battles.size();
    }

    /**
     * Returns the battle at the given index.
     *
     * @param index the index of the battle to return.
     * @return the battle at the given index.
     * @throws ArrayIndexOutOfBoundsException if the given index is negative or
     * greater than or equal to the number of battles.
     */
    public Battle getBattle(int index)
    {
	return this.battles.get(index);
    }

    /**
     * Adds the given battle to this end of the list of battles.
     *
     * @param battle the battle to add.
     */
    public void add(Battle battle)
    {
	this.battles.add(battle);
    }

    /**
     * Executes the simulation and prints the results to standard output.
     * Running this method will modify the states of abilities and GFs.
     */
    public void execute()
    {
	this.outputOpening();
	this.outputBattles();
	System.out.println();
	this.outputSummary();
    }

    /**
     * Outputs any GFs that are acquired in the opening, before the first
     * battle.
     */
    private void outputOpening()
    {
	boolean inOpening = false;
	for (GuardianForce gf : this.gfs) {
	    if (gf.acquiredInOpening()) {
		if (inOpening == false)
		    System.out.println("Opening");
		inOpening = true;
		System.out.println("    - Acquired " + gf.getName());
	    }
	}
    }

    /**
     * Outputs the result of all the battles in this simulation.
     */
    private void outputBattles()
    {
	int numBattles = this.battles.size();
	for (int onBattle = 0; onBattle < numBattles; onBattle++) {

	    /* Print battle header */
	    Battle battle = this.battles.get(onBattle);
	    int ap = battle.getAP();
	    System.out.println("Battle " + (onBattle + 1) + ": " +
			       battle.getName() + " - " + ap + " AP");

	    /*
	     * Test if we can menu before this battle.  If so, update what
	     * abilities the GFs are currently learning.
	     */
	    if (battle.canMenuBefore()) {
		for (GuardianForce gf : this.gfs) {
		    if (gf.acquiredBy(onBattle) == false)
			continue;
		    Ability current = gf.getCurrent();
		    if (current == null || current.isLearned())
			gf.nextAbility();
		}
	    }
	    else
		System.out.println("    - Cannot use menu before this battle");

	    /* Pass out the reward AP to the GFs, and output results. */
	    for (GuardianForce gf : this.gfs) {
		if (gf.acquiredBy(onBattle) == false) {
		    if (gf.acquiredIn(onBattle))
			System.out.println("    - Acquired " +
					   gf.getName());
		    continue;
		}
		if (gf.inBattle(onBattle) == false) {
		    System.out.println("    - Excludes " + gf.getName());
		    continue;
		}
		Ability current = gf.getCurrent();
		boolean doneCurrent = (current == null || current.isLearned());
		if (gf.isNextAbility() && doneCurrent) {
		    System.out.println("    - Wasted AP by " + gf.getName());
		    gf.addWasted(ap);
		}
		else if (doneCurrent == false) {
		    boolean learned = current.earn(ap);
		    if (learned) {
			int waste =
			    current.getEarnedAP() - current.getRequiredAP();
			System.out.println("    - " + gf.getName() +
					   " learned " + current.getName() +
					   " (" + waste + " AP wasted)");
			gf.incrementLearned();
			gf.addWasted(waste);
		    }
		}
		else
		    gf.addUnused(ap);
	    }
	}
    }

    /**
     * Prints a summary of all of the GF learning, at the end of the
     * simulation.
     */
    private void outputSummary()
    {
	int sum = 0;
	for (Battle battle : this.battles)
	    sum += battle.getAP();
	System.out.println("Total AP received: " + sum);

	for (GuardianForce gf : this.gfs) {
	    if (gf.isAcquired() == false) {
		System.out.println(gf.getName() + ": not acquired!");
		continue;
	    }
	    int shortBy = gf.getTotalRequired();
	    System.out.print(gf.getName() + ": " + gf.getLearned() +
			     " abilities learned, ");
	    if (shortBy > 0)
		System.out.print("short by " + shortBy + " AP, ");
	    else
		System.out.print("" + gf.getUnused() + " AP unused, ");
	    System.out.println("" + gf.getWasted() + " AP wasted");
	}
    }
}